@@ -30,6 +30,7 @@
 #include <cassert>
 #include <climits>
 #include <iostream>
+#include <stdexcept>
 
 // GNU crypto library
 #include <gcrypt.h>
@@ -66,7 +67,7 @@ unsigned long int tmcg_mpz_grandom_ui_nomodbias
 	unsigned long int div, max, rnd = 0;
 	
 	if ((modulo == 0) || (modulo == 1))
-	    return 0; // indicates an error FIXME: throw an exception
+		throw std::invalid_argument("tmcg_mpz_grandom_ui_nomodbias: bad modulo");
 	
 	// Remove ``modulo bias'' by limiting the return values
 	div = (ULONG_MAX - modulo + 1) / modulo;
@@ -119,83 +120,21 @@ unsigned long int tmcg_mpz_wrandom_mod
 void tmcg_mpz_grandomb
 	(mpz_ptr r, const unsigned long int size, enum gcry_random_level level)
 {
-	unsigned char *rtmp;
-	size_t hlen = 0;
-	gcry_mpi_t rr;
-	gcry_error_t ret;
-	assert(size <= UINT_MAX);
-
-	rr = gcry_mpi_new((unsigned int)size);
-	gcry_mpi_randomize(rr, (unsigned int)size, level);
-	ret = gcry_mpi_aprint(GCRYMPI_FMT_HEX, &rtmp, &hlen, rr);
-	gcry_mpi_release(rr);
-	if (ret || (hlen == 0))
-	{
-		std::cerr << "tmcg_mpz_grandomb(): gcry_mpi_aprint() failed: " <<
-			gcry_strerror(ret) << std::endl;
-		mpz_set_ui(r, 0L); // indicates an error
-		// FIXME: throw an exception
-	}
-	else
-	{
-		char htmp[size + 3]; // at least two characters + delimiter
-		memset(htmp, 0, size + 3);
-		memcpy(htmp, rtmp, hlen);
-		gcry_free(rtmp);
-		mpz_set_str(r, htmp, 16);
+	unsigned char tmp[(size+7)/8];
+	gcry_randomize(tmp, (size+7)/8, level);
+	mpz_import(r, (size+7)/8, 1, 1, 1, 0, (const void*)tmp);
 #ifdef BOTAN
-		memset(htmp, 0, size + 3);
-		std::unique_ptr<Botan::RandomNumberGenerator>
-			rng(new Botan::AutoSeeded_RNG);
-		uint8_t botan_tmp[hlen];
-		rng->randomize((uint8_t*)&botan_tmp, sizeof(botan_tmp));
-		for (size_t i = 0; i < hlen; i++)
-		{
-			switch (botan_tmp[i] % 16)
-			{
-				case 0:
-					htmp[i] = '0';
-				case 1:
-					htmp[i] = '1';
-				case 2:
-					htmp[i] = '2';
-				case 3:
-					htmp[i] = '3';
-				case 4:
-					htmp[i] = '4';
-				case 5:
-					htmp[i] = '5';
-				case 6:
-					htmp[i] = '6';
-				case 7:
-					htmp[i] = '7';
-				case 8:
-					htmp[i] = '8';
-				case 9:
-					htmp[i] = '9';
-				case 10:
-					htmp[i] = 'a';
-				case 11:
-					htmp[i] = 'b';
-				case 12:
-					htmp[i] = 'c';
-				case 13:
-					htmp[i] = 'd';
-				case 14:
-					htmp[i] = 'e';
-				case 15:
-					htmp[i] = 'f';
-			}
-		}
-		mpz_t rrr;
-		mpz_init(rrr);
-		mpz_set_str(rrr, htmp, 16);
-		mpz_add(r, r, rrr); // ADD number from other random source
-		mpz_clear(rrr);
+	std::unique_ptr<Botan::RandomNumberGenerator>
+		rng(new Botan::AutoSeeded_RNG);
+	rng->randomize((uint8_t*)tmp, (size+7)/8);
+	mpz_t rrr;
+	mpz_init(rrr);
+	mpz_import(rrr, size, 1, 1, 1, 0, (const void*)tmp);
+	mpz_add(r, r, rrr); // ADD number from other random source
+	mpz_clear(rrr);
 #endif
-		// r mod 2^size, i.e. shift right and bit mask
-		mpz_tdiv_r_2exp(r, r, size);
-	}
+	// r mod 2^size, i.e. shift right and bit mask
+	mpz_tdiv_r_2exp(r, r, size);
 }
 
 void tmcg_mpz_ssrandomb
@@ -233,83 +172,21 @@ void tmcg_mpz_grandomm
 	(mpz_ptr r, mpz_srcptr m, enum gcry_random_level level)
 {
 	// make bias negligible cf. BSI TR-02102-1, B.4 Verfahren 2
-	unsigned long int size = mpz_sizeinbase(m, 2L) + 64;
-	unsigned char *rtmp;
-	char htmp[size + 3]; // at least two characters + delimiter
-	size_t hlen = 0;
-	gcry_mpi_t rr;
-	gcry_error_t ret;
-	assert(size <= UINT_MAX);
-	
-	rr = gcry_mpi_new((unsigned int)size);
-	gcry_mpi_randomize(rr, (unsigned int)size, level);
-	ret = gcry_mpi_aprint(GCRYMPI_FMT_HEX, &rtmp, &hlen, rr);
-	gcry_mpi_release(rr);
-	if (ret || (hlen == 0))
-	{
-		std::cerr << "tmcg_mpz_grandomm(): gcry_mpi_aprint() failed: " <<
-			gcry_strerror(ret) << std::endl;
-		mpz_set_ui(r, 0L); // indicates an error
-		// FIXME: throw an exception
-	}
-	else
-	{
-		memset(htmp, 0, size + 3);
-		memcpy(htmp, rtmp, hlen);
-		gcry_free(rtmp);
-		mpz_set_str(r, htmp, 16);
+	unsigned long int nbytes = (mpz_sizeinbase(m, 2UL) + 64 + 7) / 8;
+	unsigned char tmp[nbytes];
+	gcry_randomize(tmp, nbytes, level);
+	mpz_import(r, nbytes, 1, 1, 1, 0, (const void*)tmp);
 #ifdef BOTAN
-		memset(htmp, 0, size + 3);
-		std::unique_ptr<Botan::RandomNumberGenerator>
-			rng(new Botan::AutoSeeded_RNG);
-		uint8_t botan_tmp[hlen];
-		rng->randomize((uint8_t*)&botan_tmp, sizeof(botan_tmp));
-		for (size_t i = 0; i < hlen; i++)
-		{
-			switch (botan_tmp[i] % 16)
-			{
-				case 0:
-					htmp[i] = '0';
-				case 1:
-					htmp[i] = '1';
-				case 2:
-					htmp[i] = '2';
-				case 3:
-					htmp[i] = '3';
-				case 4:
-					htmp[i] = '4';
-				case 5:
-					htmp[i] = '5';
-				case 6:
-					htmp[i] = '6';
-				case 7:
-					htmp[i] = '7';
-				case 8:
-					htmp[i] = '8';
-				case 9:
-					htmp[i] = '9';
-				case 10:
-					htmp[i] = 'a';
-				case 11:
-					htmp[i] = 'b';
-				case 12:
-					htmp[i] = 'c';
-				case 13:
-					htmp[i] = 'd';
-				case 14:
-					htmp[i] = 'e';
-				case 15:
-					htmp[i] = 'f';
-			}
-		}
-		mpz_t rrr;
-		mpz_init(rrr);
-		mpz_set_str(rrr, htmp, 16);
-		mpz_add(r, r, rrr); // ADD number from other random source
-		mpz_clear(rrr);
+	std::unique_ptr<Botan::RandomNumberGenerator>
+		rng(new Botan::AutoSeeded_RNG);
+	rng->randomize((uint8_t*)tmp, nbytes);
+	mpz_t rrr;
+	mpz_init(rrr);
+	mpz_import(rrr, nbytes, 1, 1, 1, 0, (const void*)tmp);
+	mpz_add(r, r, rrr); // ADD number from other random source
+	mpz_clear(rrr);
 #endif
-		mpz_mod(r, r, m); // bias is negligible due to increased size of r
-	}
+	mpz_mod(r, r, m); // bias is negligible due to increased size of r
 }
 
 void tmcg_mpz_ssrandomm
@@ -322,7 +199,7 @@ void tmcg_mpz_ssrandomm
 		if (fscanf(fhd, "%lu", &entropy_avail) != 1)
 			entropy_avail = 0;
 		fclose(fhd);
-		if (entropy_avail < mpz_sizeinbase(m, 2L))
+		if (entropy_avail < mpz_sizeinbase(m, 2UL))
 		{
 			std::cerr << "tmcg_mpz_ssrandomm(): too few entropy (" <<
 				entropy_avail << " bits) available; blocking" << std::endl;
@@ -345,31 +222,33 @@ void tmcg_mpz_wrandomm
 
 void tmcg_mpz_ssrandomm_cache_init
 	(mpz_t ssrandomm_cache[TMCG_MAX_SSRANDOMM_CACHE],
-	mpz_ptr ssrandomm_cache_mod,
-	size_t *ssrandomm_cache_avail,
-	size_t n, mpz_srcptr m)
+	 mpz_ptr ssrandomm_cache_mod,
+	 size_t &ssrandomm_cache_avail,
+	 const size_t n,
+	 mpz_srcptr m)
 {
 	size_t i = 0;
 	if ((n == 0) || (n > TMCG_MAX_SSRANDOMM_CACHE))
-		return; // FIXME: throw an exception
+		throw std::invalid_argument("tmcg_mpz_ssrandomm_cache_init: bad n");
 	for (i = 0; i < TMCG_MAX_SSRANDOMM_CACHE; i++)
 		mpz_init(ssrandomm_cache[i]);
 	for (i = 0; i < n; i++)
 		tmcg_mpz_ssrandomm(ssrandomm_cache[i], m);
 	mpz_init_set(ssrandomm_cache_mod, m);
-	*ssrandomm_cache_avail = n;
+	ssrandomm_cache_avail = n;
 }
 
 void tmcg_mpz_ssrandomm_cache
 	(mpz_t ssrandomm_cache[TMCG_MAX_SSRANDOMM_CACHE],
-	mpz_srcptr ssrandomm_cache_mod,
-	size_t *ssrandomm_cache_avail,
-	mpz_ptr r, mpz_srcptr m)
+	 mpz_srcptr ssrandomm_cache_mod,
+	 size_t &ssrandomm_cache_avail,
+	 mpz_ptr r,
+	 mpz_srcptr m)
 {
-	if (!mpz_cmp(m, ssrandomm_cache_mod) && *ssrandomm_cache_avail)
+	if (!mpz_cmp(m, ssrandomm_cache_mod) && (ssrandomm_cache_avail > 0))
 	{
-		(*ssrandomm_cache_avail)--; // next cached random value
-		mpz_set(r, ssrandomm_cache[*ssrandomm_cache_avail]);
+		ssrandomm_cache_avail--; // next cached random value
+		mpz_set(r, ssrandomm_cache[ssrandomm_cache_avail]);
 	}
 	else
 		tmcg_mpz_ssrandomm(r, m);
@@ -377,11 +256,11 @@ void tmcg_mpz_ssrandomm_cache
 
 void tmcg_mpz_ssrandomm_cache_done
 	(mpz_t ssrandomm_cache[TMCG_MAX_SSRANDOMM_CACHE],
-	mpz_ptr ssrandomm_cache_mod,
-	size_t *ssrandomm_cache_avail)
+	 mpz_ptr ssrandomm_cache_mod,
+	 size_t &ssrandomm_cache_avail)
 {
 	size_t i = 0;
-	*ssrandomm_cache_avail = 0;
+	ssrandomm_cache_avail = 0;
 	mpz_clear(ssrandomm_cache_mod);
 	for (i = 0; i < TMCG_MAX_SSRANDOMM_CACHE; i++)
 		mpz_clear(ssrandomm_cache[i]);
